<?php
/**
 * A command for user login
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 Ship-Simu Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class WebShipsimuUserLoginCommand extends BaseCommand implements Commandable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this command and sets the resolver instance
	 *
	 * @param	$resolverInstance	An instance of a command resolver
	 * @return	$commandInstance	The created command instance
	 */
	public final static function createWebShipsimuUserLoginCommand (CommandResolver $resolverInstance) {
		// Get a new instance
		$commandInstance = new WebShipsimuUserLoginCommand();

		// Set the resolver instance
		$commandInstance->setResolverInstance($resolverInstance);

		// Return the prepared instance
		return $commandInstance;
	}

	/**
	 * Executes the command with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// First get a UserLogin instance
		$loginInstance = ObjectFactory::createObjectByConfiguredName('user_login_class');

		// First set request and response instance
		$loginInstance->setRequestInstance($requestInstance);

		// Encrypt the password
		$loginInstance->encryptPassword('pass');

		// Do the login here
		$loginInstance->doLogin($requestInstance, $responseInstance);

		// Was the login fine? Then redirect here
		if ($loginInstance->ifLoginWasSuccessfull()) {
			// Try to redirect here
			try {
				// Redirect...
				$responseInstance->redirectToConfiguredUrl('app_login');

				// Exit here
				exit();
			} catch (FrameworkException $e) {
				// Something went wrong here!
				$responseInstance->addFatalMessage($e->getMessage());
			}
		} else {
			// Attach error message to the response
			$responseInstance->addFatalMessage('failed_user_login');
		}
	}

	/**
	 * Adds extra filters to the given controller instance
	 *
	 * @param	$controllerInstance		A controller instance
	 * @param	$requestInstance		An instance of a class with an Requestable interface
	 * @return	void
	 * @todo	Add more filters
	 */
	public function addExtraFilters (Controller $controllerInstance, Requestable $requestInstance) {
		// Which login type do we have?
		switch ($this->getConfigInstance()->getConfigEntry('login_type')) {
			case 'username': // Login via username
				$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('username_verifier_filter'));
				break;

			case 'email': // Login via email
				$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('email_verifier_filter'));
				break;

			default: // Wether username or email is set
				$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('username_email_verifier_filter'));
				break;
		}

		// Password verifier filter
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('password_verifier_filter'));

		// Add filter for CAPTCHA
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('captcha_user_verifier_filter'));
	}
}

// [EOF]
?>
